@@ -63,6 +63,18 @@ try:
         def CreationDate(self):
             return self.creationTime().ISO()
 
+        security.declarePublic( 'getPageTitle' )
+        def getPageTitle(self, here=None, template=None, portal_title=None):
+            """
+            Return the proper name of this page for use by Plone.
+
+            A quick fix for plone's script, which doesn't like using
+            our page's title when it's the same as the id.
+            XXX file a bug
+            """
+            return self.pageName()
+        
+
     InitializeClass(ZwikiDublinCoreImpl)
 
     class CMFAwareness(PortalContent, ZwikiDublinCoreImpl):
@@ -109,23 +121,15 @@ try:
         def SearchableText(self):
             return self.text()
 
-        #security.declareProtected(Permissions.View, '__call__')
-        #def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
-        #    '''
-        #    Invokes the default view.
-        #    '''
-        #    view = _getViewFor( self )
-        #    if getattr(aq_base(view), 'isDocTemp', 0):
-        #        return apply(view, (self, REQUEST))
-        #    else:
-        #        if REQUEST:
-        #            kw[ 'REQUEST' ] = REQUEST
-        #        if RESPONSE:
-        #            kw[ 'RESPONSE' ] = RESPONSE
-        #        return apply( view, (self,), kw )
+        security.declareProtected(Permissions.View, 'Subject')
+        def Subject(self):
+            #creates too many keywords (#1059)
+            #return self.spacedPageName()
+            return ''
 
-        # needed ?
-        #index_html = None  # This special value informs ZPublisher to use __call__
+        security.declareProtected(Permissions.View, 'Description')
+        def Description(self):
+            return self.summary()
 
         def wiki_context(self, REQUEST=None, with_siblings=0):
             return self.context(REQUEST, with_siblings)
