@@ -104,15 +104,21 @@ factory_type_information = (
                   'condition': 'object/isMailoutEnabled',
                   'category': 'object',
                   },
+                 {'id': 'recentchanges',
+                  'name': 'Recent changes',
+                  'action': 'python:object.changesUrl()',
+                  'permissions': (Permissions.View,),
+                  'category': 'object_actions',
+                  },
                  {'id': 'contents',
                   'name': 'Wiki contents',
                   'action': 'python:object.contentsUrl()',
                   'permissions': (Permissions.View,),
                   'category': 'object_actions',
                   },
-                 {'id': 'recentchanges',
-                  'name': 'Wiki changes',
-                  'action': 'python:object.changesUrl()',
+                 {'id': 'searchwiki',
+                  'name': 'Wiki search',
+                  'action': 'python:object.searchUrl()',
                   'permissions': (Permissions.View,),
                   'category': 'object_actions',
                   },
@@ -123,19 +129,6 @@ factory_type_information = (
                   'condition': 'object/hasIssues',
                   'category': 'object_actions',
                   },
-                 {'id': 'filterissues',
-                  'name': 'Filter issues',
-                  'action': 'python:object.filterUrl()',
-                  'permissions': (Permissions.View,),
-                  'condition': 'object/hasIssues',
-                  'category': 'object_actions',
-                  },
-                 {'id': 'searchwiki',
-                  'name': 'Search this wiki',
-                  'action': 'python:object.searchUrl()',
-                  'permissions': (Permissions.View,),
-                  'category': 'object_actions',
-                  },
                  {'id': 'useroptions',
                   'name': 'Wiki options',
                   'action': 'python:object.preferencesUrl()',
