@@ -7,6 +7,7 @@ from string import split,join,find,lower,rfind,atoi,strip
 from urllib import quote, unquote
 from types import *
 from email.Message import Message
+from copy import deepcopy
 
 import ZODB # need this for pychecker
 from AccessControl import getSecurityManager, ClassSecurityInfo
@@ -18,8 +19,8 @@ from OFS.DTMLDocument import DTMLDocument
 from OFS.ObjectManager import BadRequestException
 import OFS.Image
 
-from Defaults import DISABLE_JAVASCRIPT, LARGE_FILE_SIZE, \
-     ALLOWED_PAGE_TYPES, ALLOWED_PAGE_TYPES_IN_PLONE, LEAVE_PLACEHOLDER
+from pagetypes import PAGETYPES
+from Defaults import DISABLE_JAVASCRIPT, LARGE_FILE_SIZE, LEAVE_PLACEHOLDER
 import Permissions
 from Regexps import javascriptexpr, htmlheaderexpr, htmlfooterexpr
 from Utils import BLATHER, parseHeadersBody
@@ -34,9 +35,8 @@ class EditingSupport:
         return getSecurityManager().checkPermission(permission,object)
 
     security.declareProtected(Permissions.Add, 'create') 
-    def create(self,page,text=None,type=None,title='',REQUEST=None,log='',
-               leaveplaceholder=1, updatebacklinks=1, sendmail=1,
-               parents=None, subtopics=None):
+    def create(self,page,text='',type=None,title='',REQUEST=None,log='',
+               sendmail=1, parents=None, subtopics=None):
         """
         Create a new wiki page, with optional extras.
 
@@ -71,6 +71,7 @@ class EditingSupport:
         newid = self.folder()._setObject(id,p)
         p = getattr(self.folder(),newid)
 
+        p.checkForSpam(text)
         p.setCreator(REQUEST)
         p.setLastEditor(REQUEST)
         p.setLastLog(log)
@@ -81,19 +82,20 @@ class EditingSupport:
 
         # choose the specified type, the default type or whatever we're allowed 
         p.setPageType(type or self.defaultPageType())
-        p.setText(text or '',REQUEST)
+        p.setText(text,REQUEST)
         p.handleFileUpload(REQUEST)
         p.handleSubtopicsProperty(subtopics,REQUEST)
         if p.autoSubscriptionEnabled(): p.subscribeThisUser(REQUEST)
         #if p.usingRegulations(): p.setRegulations(REQUEST,new=1)
         # users can alter the page name in the creation form; allow that.
-        # We do a full rename, after all the above, to make sure
+        # We do a full rename, only after all the above, to make sure
         # everything gets handled properly.  We must first commit though,
         # to get p.cb_isMoveable(). Renaming will also get us indexed.
+        # We never leave a placeholder, always update backlinks.
         if title and title != p.title_or_id():
             get_transaction().note('rename during creation')
             get_transaction().commit()
-            p.handleRename(title,leaveplaceholder,updatebacklinks,REQUEST,log)
+            p.handleRename(title,0,1,REQUEST,log)
         else:
             # we got indexed after _setObject, but do it again with our text
             p.index_object()
@@ -104,7 +106,7 @@ class EditingSupport:
                 p.read(),
                 REQUEST=REQUEST,
                 subjectSuffix='',
-                subject='(new) '+log,
+                subject='(%s) %s' % (_('new'),log),
                 message_id=self.messageIdFromTime(p.creationTime())
                 )
         # and move on
@@ -142,6 +144,8 @@ class EditingSupport:
         if not username:
             username = self.usernameFrom(REQUEST)
             if re.match(r'^[0-9\.\s]*$',username): username = ''
+        subject_heading = self.cleanupText(subject_heading)
+        text = self.cleanupText(text)
         # some subtleties here: we ensure the page comment and mail-out
         # will use the same message-id, and the same timestamp if possible
         # (helps threading & debugging)
@@ -158,18 +162,20 @@ class EditingSupport:
         m['Subject'] = subject_heading
         m['Message-ID'] = message_id
         if in_reply_to: m['In-Reply-To'] = in_reply_to
-        m.set_payload(self.cleanupText(text))
+        m.set_payload(text)
         m.set_unixfrom(self.fromLineFrom(m['From'],m['Date'])[:-1])
         
         # discard junk comments
         if not (m['Subject'] or m.get_payload()): return
 
+        # optimisation:
         # add the comment to the page with minimal work - carefully append
         # it to both source and _prerendered cached without re-rendering
         # the whole thing! This might not be legal for all future page
         # types.
         # add to source, in standard rfc2822 format:
         t = str(m)
+        self.checkForSpam(t)
         if self.usingPurpleNumbers(): t = self.addPurpleNumbersTo(t,self)
         t = '\n\n' + t
         self.raw += t
@@ -183,7 +189,7 @@ class EditingSupport:
         self.setLastEditor(REQUEST)
         self.setLastLog(subject_heading)
         if self.autoSubscriptionEnabled(): self.subscribeThisUser(REQUEST)
-        self.updateCatalog()
+        self.index_object()
 
         # send out mail to any subscribers
         # hack the username in there for usernameFrom
@@ -253,7 +259,12 @@ class EditingSupport:
         elif self.pageWithNameOrId(page):
             p = self.pageWithNameOrId(page) # changing another page
         else:
-            return self.create(page,text,type,title,REQUEST,log,
+            return self.create(page,
+                               text or '', # string expected here
+                               type,
+                               title,
+                               REQUEST,
+                               log,
                                subtopics=subtopics) # creating a page
         if check_conflict:
             if self.checkEditConflict(timeStamp, REQUEST):
@@ -261,17 +272,22 @@ class EditingSupport:
             if self.isDavLocked():
                 return self.davLockDialog()
 
+        # XXX require at least a username cookie to edit
+        #if not self.requestHasSomeId(REQUEST):
+        #    raise 'Unauthorized', (
+        #        _('Sorry, this wiki requires that you configure a username to edit; please back up and visit options first.'))
+
         # ok, changing p. We may do several things at once; each of these
         # handlers checks permissions and does the necessary.
         if p.handleDeleteMe(text,REQUEST,log): return
         p.handleEditPageType(type,REQUEST,log)
-        p.handleEditText(text,REQUEST,subjectSuffix,log)
+        if text != None: p.handleEditText(text,REQUEST,subjectSuffix,log)
         p.handleSubtopicsProperty(subtopics,REQUEST)
         p.handleFileUpload(REQUEST,log)
         p.handleRename(title,leaveplaceholder,updatebacklinks,REQUEST,log)
         #if self.usingRegulations(): p.handleSetRegulations(REQUEST)
 
-        p.updateCatalog()
+        p.index_object()
 
         # tell browser to reload the page (or redirect elsewhere)
         if REQUEST:
@@ -332,19 +348,25 @@ class EditingSupport:
 
 
     def handleEditText(self,text,REQUEST=None, subjectSuffix='', log=''):
-        # is the new text valid and different ?
-        if (text is not None and
-            self.cleanupText(text) != self.read()):
+        old = self.read()
+        new = self.cleanupText(text)
+        # is the new text different ?
+        if new != old:
             # do we have permission ?
             if (not
                 (self.checkPermission(Permissions.Edit, self) or
                  (self.checkPermission(Permissions.Append, self)
-                  and find(self.cleanupText(text),self.read()) == 0))):
+                  and find(new,old) == 0))):
                 raise 'Unauthorized', (
                     _('You are not authorized to edit this ZWiki Page.'))
 
+            # does this edit look like spam ?
+            # Tries to count the links added by this edit. Not perfect -
+            # existing links on a line that you tweak will be counted.
+            # Not sure what happens if you replace existing links.
+            self.checkForSpam(self.addedText(old, new))
+                
             # change it
-            oldtext = self.read()
             self.setText(text,REQUEST)
             self.setLastEditor(REQUEST)
             self.setLastLog(log)
@@ -352,7 +374,7 @@ class EditingSupport:
             # if mailout policy is all edits, do it here
             if getattr(self,'mailout_policy','') == 'edits':
                 self.sendMailToSubscribers(
-                    self.textDiff(a=oldtext,b=self.read()),
+                    self.textDiff(a=old,b=self.read()),
                     REQUEST=REQUEST,
                     subject=log)
 
@@ -416,7 +438,7 @@ class EditingSupport:
         if pagename and string.strip(pagename):
             self.replaceLinksThroughoutWiki(oldname,pagename,REQUEST)
         # get parent url while we still can
-        parenturl = self.primaryParentUrl()
+        redirecturl = self.upUrl()
         # unindex (and remove from outline) and move to the recycle bin folder
         self.recycle(REQUEST)
         # notify subscribers if appropriate
@@ -426,7 +448,7 @@ class EditingSupport:
                 REQUEST=REQUEST,
                 subjectSuffix='',
                 subject='(deleted)')
-        if REQUEST: REQUEST.RESPONSE.redirect(parenturl)
+        if REQUEST: REQUEST.RESPONSE.redirect(redirecturl)
 
     def ensureRecycleBin(self):
         if not hasattr(self.folder(),'recycle_bin'):
@@ -495,9 +517,10 @@ class EditingSupport:
 
         # update wiki outline
         # changeIdPreservingCreator->manage_renameObject->_delObject/_setObject
-        # has the effect of losing our place in the hierarchy, take a snapshot
-        # so we can fix it up later
-        savedparentmap = self.wikiOutline().parentmap().copy()
+        # -> manage_after* has the effect of losing our place in the
+        # hierarchy, take a snapshot so we can fix it up later
+        savedparentmap = deepcopy(self.wikiOutline().parentmap())
+        savedchildmap = deepcopy(self.wikiOutline().childmap())
 
         # has the page id changed ?
         if newid != oldid:
@@ -512,8 +535,10 @@ class EditingSupport:
         self.title = newname
         # update catalog
         self.index_object() # XXX creator info may need reindexing too
-        # update wiki outline
+        # update wiki outline, using the copy we saved earlier
+        # nb that outline may not have been up to date, but replace will forgive
         self.wikiOutline().setParentmap(savedparentmap)
+        self.wikiOutline().setChildmap(savedchildmap)
         self.wikiOutline().replace(oldname,newname)
 
         # do this after the above so it will have correct parent
@@ -757,15 +782,45 @@ class EditingSupport:
         #    #get_transaction().commit()
         #    DTMLDocument.__call__(self,self,REQUEST,REQUEST.RESPONSE)
 
+    def checkForSpam(self, t):
+        """
+        Check for signs of spam in some text, and raise an error if found.
+
+        Also looks at the current user's info in REQUEST.
+        """
+        REQUEST = getattr(self,'REQUEST',None)
+        username = self.usernameFrom(REQUEST,ip_address=0)
+        ip = getattr(REQUEST,'REMOTE_ADDR','')
+        page = self.pageName()
+        def raiseSpamError(reason, verbose_reason):
+            BLATHER(('blocked edit from %s (%s) on %s (%s)\n%s\n') % \
+                    (ip, username, page, reason, t))
+            raise _("There was a problem: %s. Please contact the site administrator for help." % \
+                    (verbose_reason))
+            
+        # banned link pattern ?
+        for pat in getattr(self.folder(),'banned_links',[]):
+            pat = strip(pat)
+            if pat and re.search(pat,t):
+                raiseSpamError(_("banned_links match"),
+                               _("your edit contained a banned link pattern"))
+
+        # anonymous edit with too many urls ?
+        prop = 'max_anonymous_links'
+        # we'll handle either an int or string property
+        if (not self.requestHasSomeId(REQUEST) and
+            hasattr(self.folder(), prop)):
+            try: max = int(getattr(self.folder(), prop))
+            except ValueError: max = None
+            if max is not None:
+                if len(re.findall(r'https?://',t)) > max:
+                    raiseSpamError(_("exceeded max_anonymous_links"),
+                                   _("adding of external links by anonymous authors is restricted"))
+
     def cleanupText(self, t):
         """
         Do some cleanup of incoming text, also block spam links.
         """
-        # block spammer links - for now, just give an unexplained site error
-        for pat in getattr(self.folder(),'banned_links',[]):
-            pat = strip(pat)
-            if pat and re.search(pat,t): raise
-        
         # strip any browser-appended ^M's
         t = re.sub('\r\n', '\n', t)
 
@@ -940,12 +995,13 @@ class EditingSupport:
 
     def allowedPageTypes(self):
         """
-        This wiki's "allowed" page types, ie those offered in the edit form.
+        List the page type ids which may be selected in this wiki's edit form.
+
+        This will be all available page types, unless overridden by an
+        allowed_page_types property.
         """
-        if self.inCMF(): default = ALLOWED_PAGE_TYPES_IN_PLONE
-        else: default = ALLOWED_PAGE_TYPES
-        return filter(lambda x:strip(x),
-                      getattr(self,'allowed_page_types',[])) or default
+        return (filter(lambda x:strip(x),getattr(self,'allowed_page_types',[]))
+                or map(lambda x:x._id, PAGETYPES))
 
     def defaultPageType(self):
         """This wiki's default page type."""
