@@ -334,6 +334,18 @@ class SubscriberManagerMixin:
         else:
             return 0
 
+    def usernameOrEmailOfSubscriber(self):
+        """
+        if the user is logged into the CMF, return his/her username
+        else return his/her email address
+        """ 
+        username = str(self.portal_membership.getAuthenticatedMember())
+        if username and not self.portal_membership.isAnonymousUser():
+            return username
+        else:
+            return self.REQUEST.get('email')
+
+
     # utilities
 
     def emailAddressFrom(self,subscriber):
