@@ -7,6 +7,7 @@ class Outline:
     I represent and answer questions about a multi-root, multi-parent
     hierarchy of objects, usually strings, efficiently.
 
+    This is based on Ken Manheimer's WikiNesting enhancement.
     For the moment, an Outline has
       _parentmap = {'Root':[],'Child':['Root'],'GrandChild':['Child'],'Single':[]}
       _childmap =  {'Root':['Child'],'Child':['GrandChild'],'GrandChild':[],'Single':[]}
@@ -15,9 +16,8 @@ class Outline:
       Leaves: the string name of the page
       Nodes with children: a list beginning with the parent node's name
       Nodes with omitted children (for brevity): list with one string.
-    This is based on Ken Manheimer's WikiNesting implementation for Zwiki.
+    _childmap and _nesting are derived from _parentmap.
     Outlines and nestings should perhaps be the same thing.
-    childmap is the inverse of parentmap.
 
     """
     _parentmap = {}
@@ -45,15 +45,35 @@ class Outline:
     def leaves(self):
         """Return a sorted list of the leaf nodes."""
         return filter(lambda x:not self.children(x), self.nodes())
-    def updateChildmap(self):
-        """Regenerate childmap from parentmap."""
-        childmap = {}
-        for c in self.nodes():
-            parents = self.parents(c)
-            for p in parents:
+    def updateChildmap(self,reset=0):
+        """Regenerate childmap from parentmap.
+
+        childmap is the inverse of parentmap - except, it remembers any
+        manual re-ordering of children. Unless reset is true we try to
+        preserve the order of children, which complicates things badly.
+        """
+        nodes = self.nodes()
+        # XXX still problems with things not getting updated properly
+        oldchildmap = self.childmap()
+        if reset:
+            childmap = {}
+        else:
+            # backwards compatibility: 0.39 might have set this to None
+            childmap = self.childmap() or {} 
+            # remove any no-longer-existing nodes from childmap
+            for p in childmap.keys()[:]:
+                if not p in nodes: del childmap[p]
+                else:
+                    for c in childmap[p][:]: # a copy
+                        if not c in nodes: childmap[p].remove(c)
+        # make sure each existing node appears in the childmap
+        for c in nodes:
+            for p in self.parents(c):
+                # each parent should have a childmap entry including this child
                 if not childmap.has_key(p): childmap[p] = [c]
-                else: childmap[p].append(c)
-        for l in filter(lambda x:x not in childmap.keys(),self.nodes()):
+                elif not c in childmap[p]: childmap[p].append(c)
+        # add a childmap entry for any nodes we missed (non-parents)
+        for l in filter(lambda x:x not in childmap.keys(),nodes):
             childmap[l] = []
         self.setChildmap(childmap)
     def updateNesting(self):
@@ -63,7 +83,7 @@ class Outline:
         """Regenerate everything from the parentmap."""
         self.updateChildmap()
         self.updateNesting()
-    def __init__(self,parentmap):
+    def __init__(self,parentmap={}):
         self.setParentmap(parentmap)
         self.update()
     def add(self,node,parents=[],update=1):
@@ -89,34 +109,90 @@ class Outline:
     def replace(self,node,newnode,update=1):
         """
         Replace node with newnode in the outline.
+
+        If node wasn't there, just add newnode. This is useful for rename().
+        Should this sort of robustness check be done here or there ?
+
+        Tries to preserve node's ordering among it's siblings, as in
+        updateChildMap.
         """
         parentmap = self.parentmap()
         # replace node with newnode, preserving node's parents
         parents = self.parents(node)
-        del parentmap[node]
+        if parentmap.has_key(node): del parentmap[node]
         parentmap[newnode] = parents
         # reparent node's children under newnode
         for c in filter(lambda x:node in parentmap[x],parentmap.keys()):
             parentmap[c].remove(node)
             parentmap[c].append(newnode)
         self.setParentmap(parentmap)
+
+        # tweak childmap before updateChildMap, to preserve node's position
+        childmap = self.childmap()
+        for p in childmap.keys():
+            if node in childmap[p]:
+                childmap[p][childmap[p].index(node)] = newnode
+        self.setChildmap(childmap)
+
         if update: self.update()
     def reparent(self,node,newparents,update=1):
         """
         Change node's parents to newparents in the outline.
         """
+        # help prepare childmap for updateChildMap 
+        childmap = self.childmap()
+        for p in self.parents(node): childmap[p].remove(node)
+        self.setChildmap(childmap)
+            
         self.add(node,newparents,update)
-    def next(self,node):
+    def reorder(self,node,child):
+        """
+        Moves child one place to the left among node's children (in _childmap).
+        """
+        childmap = self.childmap()
+        #we do no error checking in this class IIRC
+        #if not childmap.has_key(node): return
+        children = childmap[node]
+        #if not child in children: return
+        i = children.index(child)
+        #if not i: return
+        children[i-1], children[i] = children[i], children[i-1]
+        childmap[node] = children
+        self.setChildmap(childmap)
+        self.updateNesting()
+    def first(self):
+        """
+        Get the first node in the outline.
+        """
+        list = self.flat()
+        if list: return list[0]
+        else: return None
+    def last(self):
+        """
+        Get the last node in the outline.
+        """
+        list = self.flat()
+        if list: return list[-1]
+        else: return None
+    def next(self,node,wrap=0):
+        """
+        Get the next node in the outline.
+        """
         list = self.flat()
         if node in list:
             i = list.index(node)
             if i < len(list)-1: return list[i+1]
+            elif wrap: return list[0]
         return None
-    def previous(self,node):
+    def previous(self,node,wrap=0):
+        """
+        Get the previous node in the outline.
+        """
         list = self.flat()
         if node in list:
             i = list.index(node)
             if i > 0: return list[i-1]
+            elif wrap: return list[-1]
         return None
     def ancestors(self,node):
         """
