@@ -54,6 +54,7 @@ import Acquisition
 from App.Common import absattr
 from Globals import InitializeClass
 import Persistence
+from OFS.SimpleItem import SimpleItem
 
 import Permissions
 from Utils import flatten, BLATHER
@@ -74,8 +75,7 @@ def deepappend(nesting, page):
         else: nesting[-1] = [nesting[-1],page]
     return nesting
 
-class PersistentOutline(
-    Outline.Outline, Acquisition.Implicit, Persistence.Persistent):
+class PersistentOutline(Outline.Outline, SimpleItem):
     """
     I am a persistent version of Outline.
 
@@ -109,7 +109,30 @@ class ParentsPropertyMixin:
         {'id':'parents', 'type': 'lines', 'mode': 'w'},
         )
 
+    def ensureParentsPropertyIsList(self):
+        """
+        Ensure our parents property is a list, returning true if changed.
+
+        Zope lines properties switched from tuple to list a while back,
+        and lingering tuples cause ongoing breakage.  Called by upgrade,
+        ensureValidParents and accessors for maximum robustness.
+        """
+        if type(self.parents) != ListType:
+            BLATHER("converting %s's parents property to a list" % self.pageName())
+            self.setParents(self.parents)
+            return True
+        else:
+            return False
+
+    # I'd call this parents, but too many upgrade hassles ?
+    def getParents(self):
+        """
+        Robust accessor returning a copy of our parents list.
+        """
+        return list(self.parents)
+
     def setParents(self,parents):
+        parents = list(parents)
         parents.sort()
         self.parents = parents
 
@@ -119,24 +142,21 @@ class ParentsPropertyMixin:
             # ordering hack.. 
             #parent = string.strip(parent)
             if parent and not parent in self.parents:
+                self.ensureParentsPropertyIsList()
                 self.parents.append(parent)
 
     def removeParent(self,parent):
+        self.ensureParentsPropertyIsList()
         try: self.parents.remove(parent)
         except ValueError:
             BLATHER("failed to remove %s from %s's parents (%s)" \
                  % (parent,self.getId(),self.parents))
 
-    def checkParents(self, update_outline=1):
+    def ensureValidParents(self):
         """
-        Make sure that this page's parents property is all valid.
-
-        Also updates the catalog and wiki outline (optional in case we
-        want to do a bunch of these). This is a little higher-level
-        than setParents.
+        Ensure that this page's parents are all valid, and reindex if needed.
         """
-        parents = self.parents
-        parents.sort()
+        parents = self.getParents()
         # convert to exact page names, filtering out any which don't exist
         cleanedupparents = map(lambda x:absattr(x.Title),
                                filter(lambda x:x,
@@ -145,15 +165,14 @@ class ParentsPropertyMixin:
         # make sure we're not parented under ourself
         if self.pageName() in cleanedupparents:
             cleanedupparents.remove(self.pageName())
-        # and sort
+        # sort
         cleanedupparents.sort()
-        # if it's changed, save it and update everything
+        # if changed, save and reindex
         if cleanedupparents != parents:
             BLATHER("adjusting %s's parents from %s to %s" % 
                  (self.pageName(), parents, cleanedupparents))
             self.setParents(cleanedupparents)
-            self.index_object() #XXX only need update parents index & metadata
-            if update_outline: self.updateWikiOutline()
+            self.index_object() #XXX only need to update parents index & metadata
 
 InitializeClass(ParentsPropertyMixin) 
 
@@ -182,58 +201,104 @@ class OutlineManagerMixin:
     def updateWikiOutline(self):
         """
         Regenerate the wiki folder's cached outline object.
+
+        The wiki's outline object (a PersistentOutline) is a
+        representation of the page hierarchy, containing the same
+        information as in the pages' parents properties but in a form
+        easier to query. This method either generates a new one from the
+        parents properties, or updates an old one trying to preserve the
+        order of subtopics. Complications.
+
+        This checks and corrects any invalid parents information.  It also
+        loads all page objects from the ZODB, which is probably ok as this
+        is not done frequently.
         """
-        BLATHER('saving outline data for wiki',self.folder().getId())
-        self.folder().outline = self.wikiOutlineFromParents()
+        BLATHER('regenerating outline data for wiki',self.folder().getId())
+        oldchildmap = {}
+        # backwards compatibility
+        # a pre-0.39 outline is just an attribute, delete (but save childmap)
+        if (hasattr(self.folder().aq_base,'outline') and
+            not 'outline' in self.folder().objectIds()):
+            oldchildmap = self.folder().outline.childmap()
+            del self.folder().outline
+        # if there's no outline object, make one
+        if not hasattr(self.folder().aq_base,'outline'):
+            self.folder()._setObject('outline', PersistentOutline())
+            self.folder().outline.setChildmap(oldchildmap)
+        # regenerate the parentmap
+        parentmap = {}
+        for p in self.pageObjects():
+            p.ensureValidParents() # poor caching
+            parentmap[p.pageName()] = p.getParents()
+        self.folder().outline.setParentmap(parentmap)
+        # update the childmap (without losing subtopics order) and nesting
+        self.folder().outline.update()
+
+    # easier alias ?
+    updatecontents = updateWikiOutline
         
     def wikiOutlineFromParents(self):
         """
         Generate an outline object from the pages' parents properties.
 
-        Note we don't check for valid parents here.
         """
-        parentmap = {}
-        for p in self.pages():
-            name = p.Title
-            parents = list(p.parents) # take a copy, and make sure it's a list
-            parents = filter(lambda x:x,parents) # remove stray null strings
-            parents.sort()
-            parentmap[name] = parents
         return PersistentOutline(parentmap)
 
     security.declareProtected(Permissions.Reparent, 'reparent')
-    def reparent(self, parents=None, REQUEST=None, pagename=None):
+    def reparent(self, parents=[], REQUEST=None, pagename=None):
         """
         Move this page under the named parent pages in the wiki outline.
 
         parent page names may be passed in several ways:
         - in the parents argument (a list or string of page names)
-        - in a parents REQUEST attribute (ditto) #XXX needed ?
         - in the pagename argument (a single name)
         (this last is to support the page management form).
 
         Page names may be ids, or fuzzy, even partial. Any which do not
         resolve to an existing page or are duplicates will be ignored.
         """
-        # validate args
-        if pagename: parents = [pagename]
-        if parents is None: parents = REQUEST.get('parents', [])
-        if type(parents) != ListType: parents = [parents]
-        parents = map(lambda x:absattr(x.Title),
-                      filter(lambda x:x,
-                             map(lambda x:self.pageWithFuzzyName(x,allow_partial=1),
-                                 parents)))
+        # clean the arguments carefully to avoid parenting anomalies
+        # page mgmt form must use pagename field:
+        if pagename:
+            parents = [pagename] 
+        # or parents might be a string
+        elif type(parents) != ListType:
+            parents = [parents] 
+        # empty strings are common, remove before calling pageWithFuzzyName
+        parents = filter(lambda x:x, parents)
+        # look up the page (brain) corresponding to each (fuzzy) parent name
+        parents = map(lambda x:self.pageWithFuzzyName(x,allow_partial=1),
+                      parents)
+        # strip out Nones (pages not found)
+        parents = filter(lambda x:x, parents)
+        # convert back to proper page names
+        parents = map(lambda x:absattr(x.Title), parents)
+        # remove any duplicates
         uniqueparents = []
         for p in parents:
             if not p in uniqueparents: uniqueparents.append(p)
-        # update page's parents attribute
+            
+        # finally - update our parents property, the outline cache, and catalog
         self.setParents(uniqueparents) 
-        # update wiki outline
         self.wikiOutline().reparent(self.pageName(),uniqueparents)
-        # update wiki catalog
         self.index_object()
+
         if REQUEST is not None: REQUEST.RESPONSE.redirect(REQUEST['URL1'])
 
+    security.declareProtected(Permissions.Reparent, 'reorder')
+    def reorder(self, child, REQUEST=None):
+        """
+        Move child one place to the left among this page's children.
+
+        Ordering is only stored in the wiki outline cache, and will be
+        lost if the outline is completely regenerated.
+
+        When invoked from the web, redirects to /backlinks.
+        """
+        if not child in self.childrenAsList(): return
+        self.wikiOutline().reorder(self.pageName(),child)
+        if REQUEST is not None: REQUEST.RESPONSE.redirect(REQUEST['URL1']+'/backlinks')
+
     # queries
 
     def wikiOutline(self):
@@ -242,7 +307,7 @@ class OutlineManagerMixin:
 
         We'll generate it if needed.
         """
-        if (not hasattr(self.folder(),'outline')
+        if (not hasattr(self.folder().aq_base,'outline')
             or not self.folder().outline):
             self.updateWikiOutline()
         return self.folder().outline
@@ -267,9 +332,45 @@ class OutlineManagerMixin:
         """
         p = self.primaryParent()
         if p: return p.pageUrl()
-        # going up from a top level page leads to..
-        #else: return self.wikiUrl()    # the front page
-        else: return self.contentsUrl() # the wiki contents
+        else: return None
+
+    def upUrl(self):
+        """
+        Get the URL of whatever is "above" this page.
+        """
+        return self.primaryParentUrl() or self.defaultPageUrl()
+
+    security.declareProtected(Permissions.View, 'firstPage')
+    def firstPage(self):
+        """
+        Get the name of the first page in the hierarchy.
+        """
+        return self.wikiOutline().first()
+
+    security.declareProtected(Permissions.View, 'firstPageUrl')
+    def firstPageUrl(self):
+        """
+        Get the URL of the first page in the hierarchy.
+        """
+        p = self.pageWithName(self.firstPage())
+        if p: return p.pageUrl()
+        else: return None
+
+    security.declareProtected(Permissions.View, 'lastPage')
+    def lastPage(self):
+        """
+        Get the name of the last page in the hierarchy.
+        """
+        return self.wikiOutline().last()
+
+    security.declareProtected(Permissions.View, 'lastPageUrl')
+    def lastPageUrl(self):
+        """
+        Get the URL of the last page in the hierarchy.
+        """
+        p = self.pageWithName(self.lastPage())
+        if p: return p.pageUrl()
+        else: return None
 
     security.declareProtected(Permissions.View, 'nextPage')
     def nextPage(self):
@@ -287,7 +388,7 @@ class OutlineManagerMixin:
         """
         p = self.pageWithName(self.nextPage())
         if p: return p.pageUrl()
-        else: return self.wikiUrl()
+        else: return None
 
     security.declareProtected(Permissions.View, 'previousPage')
     def previousPage(self):
@@ -303,7 +404,7 @@ class OutlineManagerMixin:
         """
         p = self.pageWithName(self.previousPage())
         if p: return p.pageUrl()
-        else: return self.wikiUrl()
+        else: return None
 
     security.declareProtected(Permissions.View, 'ancestorsAsList')
     def ancestorsAsList(self, REQUEST=None):
@@ -398,23 +499,24 @@ class SubtopicsPropertyMixin:
     """
     security = ClassSecurityInfo()
 
-    def subtopicsEnabled(self):
+    def subtopicsEnabled(self,**kw):
         """
         Decide in a complicated way if this page should display it's subtopics.
 
-        First, the folder must have a true show_subtopics property (can
-        acquire). Then, we look for another true or false show_subtopics
-        property in:
-        - REQUEST
-        - the current page
-        - our primary ancestor pages, all the way to the top,
-        and return the first one we find,
-        or default to true if it's not found in any of those places.
+        First, if the folder has a show_subtopics property (can acquire)
+        and it's false, we will never show subtopics.  Otherwise look for
+        a show_subtopics property
+        - in REQUEST
+        - on the current page
+        - on our primary ancestor pages, all the way to the top,
+        and return the first one we find. Otherwise return true.
         """
         prop = 'show_subtopics'
-        if getattr(self.folder(),prop,0):
-            if hasattr(self,'REQUEST') and hasattr(self.REQUEST,prop):
-                return getattr(REQUEST,prop) and 1
+        if getattr(self.folder(),prop,1):
+            if kw.has_key(prop):
+                return kw[prop] and 1
+            elif hasattr(self,'REQUEST') and hasattr(self.REQUEST,prop):
+                return getattr(self.REQUEST,prop) and 1
             elif hasattr(self.aq_base,prop):
                 return getattr(self,prop) and 1
             elif self.primaryParent():
@@ -509,7 +611,7 @@ class OutlineRenderingMixin:
                     '<a href="%s/%s" name="%s" title="%s">%s</a>'\
                     % (baseurl, self.canonicalIdFrom(i), quote(i),
                        linktitle,
-                       i))
+                       self.formatWikiname(i)))
             else:
                 combos.append(i)
         here = (here and unquote(here)) or self.pageName()
@@ -543,7 +645,7 @@ class OutlineRenderingMixin:
         # format and link it
         # backwards compatibility: in case of an old editform template
         # which shows context, include the new page name at the bottom (unlinked)
-        if REQUEST and REQUEST.has_key('page') and REQUEST['page'] is not here:
+        if REQUEST and REQUEST.has_key('page') and REQUEST['page'] != here:
             here = REQUEST['page']
             nesting = deepappend(nesting, here)
             suppress_hyperlink=1
@@ -697,17 +799,17 @@ class OutlineRenderingMixin:
             return ''
         if did is None: did = []
         if got is None:
-            got = ['<ul>']
+            got = ['<ul class="aqtree3clickable">'] #SKWM 
             recursing = 0
         else:
             recursing = 1
         for n in nesting:
             if type(n) == ListType:
                 if not (n[0]==here and suppress_current): #XXX temp
-                    got.append('%s <li>[%s]</li>' % (indent,n[0]))
+                    got.append('%s <li>[%s]' % (indent,n[0]))
                 if len(n) > 1:
                     if not (n[0]==here and suppress_current): #XXX temp
-                        got.append("<ul>")
+                        got.append('<ul>')
                     for i in n[1:]:
                         if type(i) == ListType:
                             got = self.renderNesting(
@@ -718,6 +820,8 @@ class OutlineRenderingMixin:
                         got.append("</ul>")
                 else:
                     got[-1] += ' ...' # a parent whose children were omitted
+                if not (n[0]==here and suppress_current):
+                    got.append('%s </li>' % indent)
             else:
                 got.append('%s <li>[%s]</li>' % (indent,n))
         if recursing: return got
@@ -735,7 +839,7 @@ class OutlineRenderingMixin:
                            '<a href="%s/backlinks" title="%s">%s</a>' % (
                                self.page_url(),
                                _("which pages link to this one ?"),
-                               self.pageName()),
+                               self.formatWikiname(self.pageName())),
                            t)
             elif SHOW_CURRENT_PAGE_IN_CONTENTS:
                 t = re.sub(r'(\[%s\])' % re.escape(here),
@@ -748,7 +852,8 @@ class OutlineRenderingMixin:
         def quicklink(match):
             page = match.group(1)
             return '<a href="%s/%s" name="%s">%s</a>' \
-                   % (wikiurl,self.canonicalIdFrom(page),quote(page),page)
+                   % (wikiurl,self.canonicalIdFrom(page),quote(page),
+                      self.formatWikiname(page))
         t = re.sub(bracketedexpr,quicklink,t)
         return t
 
