@@ -112,6 +112,9 @@ def onlyBodyFrom(t):
     # maybe better, but more inclined to mess with valid text ?
     #return re.sub(htmlbodyexpr, r'\1', t)
 
+def addErrorTo(text,error):
+    return """<div class="error">%s</div>\n%s""" % (error,text)
+
 
 DEFAULT_TEMPLATES = {
     'badtemplate'        : loadPageTemplate('badtemplate'),
@@ -132,6 +135,8 @@ DEFAULT_TEMPLATES = {
     'issuetrackerdtml'   : loadDtmlMethod('issuetrackerdtml'),
     'filterissues'       : loadPageTemplate('filterissues'),
     'filterissuesdtml'   : loadDtmlMethod('filterissuesdtml'),
+    'issuebrowser'       : loadPageTemplate('issuebrowser'),
+    'issuebrowserdtml'   : loadDtmlMethod('issuebrowserdtml'),
     'stylesheet'         : loadStylesheetFile('stylesheet.css'),
     }
 
@@ -205,8 +210,11 @@ class UIUtils:
             return body
 
         folder = self.folder()
-        if hasattr(folder,'wikipage') and isPageTemplate(folder.wikipage):
-            return folder.wikipage.__of__(self)(self,REQUEST,body=body,**kw)
+        skintemplate = kw.get('skin','wikipage')
+        if (hasattr(folder,skintemplate) and
+            isPageTemplate(getattr(folder,skintemplate))):
+            return getattr(folder,skintemplate).__of__(self)(
+                self,REQUEST,body=body,**kw)
         elif (hasattr(folder,'standard_wiki_header') and
               hasattr(folder,'standard_wiki_footer')):
             return self.getSkinTemplate('standard_wiki_header')(self,REQUEST)+\
@@ -242,6 +250,68 @@ class UIUtils:
     #    else:
     #        return '<p>This wiki has a custom standard_wiki_header but no corresponding standard_wiki_footer. Suggestion: remove it.</body></html>'
 
+    security.declareProtected(Permissions.View, 'setSkinMode')
+    def setSkinMode(self,REQUEST,mode):
+        """
+        Change the user's display mode cookie for the standard skin.
+        
+        The display mode affects the appearance of Zwiki's standard skin;
+        it may be full, simple, or minimal.
+
+        XXX nb keep useroptions synced with this.
+        """
+        RESPONSE = REQUEST.RESPONSE
+        RESPONSE.setCookie('zwiki_displaymode',
+                           mode,
+                           path='/',
+                           expires=(self.ZopeTime() + 365).rfc822()) # 1 year
+        REQUEST.RESPONSE.redirect(REQUEST.get('URL1'))
+
+    security.declareProtected(Permissions.View, 'setCMFSkin')
+    def setCMFSkin(self,REQUEST,skin):
+        """
+        Change the user's CMF/Plone skin preference, if possible.
+        """
+        # are we in a CMF site ?
+        if not self.inCMF(): return
+        portal_skins = self.portal_url.getPortalObject().portal_skins
+        portal_membership = self.portal_url.getPortalObject().portal_membership
+        # does the named skin exist ?
+        def hasSkin(s): return portal_skins.getSkinPath(s) != s
+        if not hasSkin(skin): return
+        # is the user logged in ? 
+        member = portal_membership.getAuthenticatedMember()
+        if not hasattr(member,'setProperties'): return
+        # change their skin preference and reload page
+        REQUEST.form['portal_skin'] = skin
+        member.setProperties(REQUEST)
+        portal_skins.updateSkinCookie()
+        REQUEST.RESPONSE.redirect(REQUEST.get('URL1'))
+
+    security.declareProtected(Permissions.View, 'setSkin')
+    def setskin(self,REQUEST,skin=None):
+        """
+        Change the user's skin, or skin display mode.
+
+        skin can be either a display mode of Zwiki's standard skin - full,
+        simple, minimal - or the name of a CMF/Plone skin, or just plone.
+        (standard skin modes can work in cmf/plone, if there is a skin
+        named "Zwiki" with the "standard" layer above "zwiki_plone".)
+
+        Calling this with no arguments will select the standard skin's
+        simple mode, or the Plone Default skin if you are in CMF.
+        """
+        # convenient defaults:
+        if not skin or skin in ['plone','cmf']:
+            if self.inCMF(): skin = 'Plone Default'
+            else: skin = 'simple'
+        RESPONSE = REQUEST.RESPONSE
+        if skin in ['full','simple','minimal']:
+            self.setSkinMode(REQUEST,skin)
+            self.setCMFSkin(REQUEST,'Zwiki')
+        else:
+            self.setCMFSkin(REQUEST,skin)
+
 InitializeClass(UIUtils)
 
 
@@ -256,15 +326,15 @@ class GeneralForms:
     security.declareProtected(Permissions.View, 'wikipage')
     def wikipage(self, dummy=None, REQUEST=None, RESPONSE=None):
         """
-        Render the main page view (template-customizable).
+        Render the main page view (dummy method to allow standard skin in CMF).
 
+        XXX this may or may not still be useful. Old comment:
         The wikipage template is usually applied by addSkinTo;
         this is provided so you can configure it as the "view" method
         in portal_types -> Wiki Page -> actions to use Zwiki's standard 
         skin inside a CMF/Plone site.
         """
-        return self.getSkinTemplate('wikipage')(self,REQUEST,
-                                                body=self.render())
+        return self.render(REQUEST=REQUEST,RESPONSE=RESPONSE)
 
     # backwards compatibility
     wikipage_view = wikipage
@@ -485,6 +555,22 @@ class GeneralForms:
         else:
             return form(self,REQUEST)
 
+    security.declareProtected(Permissions.View, 'issuebrowser')
+    def issuebrowser(self, REQUEST=None):
+        """
+        Render the issuebrowser form (template-customizable).
+
+        The default page template calls a DTML helper template,
+        for ease of syncing with the evolving wiki-page-based version.
+        """
+        form = self.getSkinTemplate('issuebrowser')
+        dtmlpart = self.getSkinTemplate('issuebrowserdtml')
+        # XXX kludge - a customized template may not require our dtml part
+        if re.search(r'"structure options/body',form.read()):
+            return form(self,REQUEST,body=dtmlpart(self,REQUEST))
+        else:
+            return form(self,REQUEST)
+
     security.declareProtected(Permissions.View, 'editConflictDialog')
     def editConflictDialog(self):
         """
@@ -557,6 +643,12 @@ class GeneralForms:
         Show the access keys supported by the built-in skins.
         """
         return _("""
+        Access keys can be accessed in mozilla-based browsers by pressing alt-<key>
+        IE users: must also press enter
+        Mac users: command-<key>
+        Opera users: shift-escape-<key>
+        These won't work here, back up to the previous page to try them out.
+
         0    show these access key assignments
 
         wiki functions:
@@ -571,10 +663,11 @@ class GeneralForms:
         s    go to search field
         
         page functions:
-        v    view page
-        +    view pages in full mode
-        =    view pages in simple mode
-        -    view pages in minimal mode
+        +    view pages in plone/cmf skin (in a plone/cmf site)
+        =    view pages in standard skin, full mode
+        _    view pages in standard skin, simple mode
+        -    view pages in standard skin, minimal mode
+        v    view page without changing display mode
         m    mail subscription
         b    show backlinks (links to this page)
         d    show diffs (page edit history)
@@ -593,6 +686,7 @@ class GeneralForms:
         
         in edit form:
         s    save changes
+        p    preview
         
         when viewing diffs:
         n    next edit
