@@ -60,7 +60,7 @@ class Utils:
         strings = filter(lambda x:type(x)==type(''), strings)
         return len(join(strings,''))
 
-    def summary(self,size=100,paragraphs=1):
+    def summary(self,size=200,paragraphs=1):
         """
         Give a short summary of this page's content.
 
@@ -76,29 +76,28 @@ class Utils:
         return html_quote(t)
 
     security.declareProtected(Permissions.View, 'excerptAt')
-    def excerptAt(self, expr, size=100, highlight=1):
+    def excerptAt(self, expr, size=100, highlight=1, text=None):
         """
-        Return an excerpt of this page at the first occurence of expr.
+        Return a highlighted search result excerpt from this page (or text).
 
-        Useful when presenting search results.  Does a straight
-        case-insensitive string search, after first removing any *
-        wildcard character (this may not agree exactly with the catalog
-        index's search). SGML tags are quoted.
+        This method searches this page's text, or the provided text, for
+        the first occurrence of expr (cleaned up) and returns the
+        surrounding text chunk, html quoted, and optionally with the
+        matches enclosed in styled spans. If no match is found, it just
+        returns a chunk from the beginning.
 
-        If no match is found or expr is blank, returns an empty string.
-
-        When highlight is true, each match within the excerpt is enclosed
-        in a span with class "hit". This is not 100% reliable because of
-        html quoting and because we don't handle wildcards.
-        
+        It should mimic the search strategy of SearchPage, but that
+        depends on catalog configuration and it currently doesn't do a
+        very good job, so the excerpts and highlights can be misleading.
         """
+        text = text or self.text()
         string = re.sub(r'\*','',expr)
-        m = re.search(r'(?i)'+re.escape(string),self.text())
+        m = re.search(r'(?i)'+re.escape(string),text)
         if m and string:
             middle = (m.start()+m.end())/2
             exstart = max(middle-size/2-1,0)
-            exend = min(middle+size/2+1,len(self.text()))
-            excerpt = html_quote(self.text()[exstart:exend])
+            exend = min(middle+size/2+1,len(text))
+            excerpt = html_quote(text[exstart:exend])
             if highlight:
                 excerpt = re.sub(
                     r'(?i)'+re.escape(html_quote(string)),
@@ -108,7 +107,7 @@ class Utils:
                     excerpt)
             return excerpt
         else:
-            return ''
+            return html_quote(text[:size])
 
     def metadataFor(self,page):
         """
@@ -242,7 +241,11 @@ class Utils:
         authenticated_name = None
         user = REQUEST.get('AUTHENTICATED_USER')
         if user:
+            #if self.inCMF():
+            #  authenticated_name = self.folder().portal_membership.getMemberById(user.getUserName()).fullname or ''
+            #else:
             authenticated_name = user.getUserName()
+            
             if authenticated_name == str(user.acl_users._nobody):
                 authenticated_name = None
         cookie_name = REQUEST.cookies.get('zwiki_username',None)
@@ -265,20 +268,30 @@ class Utils:
         return ("<%s %s at 0x%s>"
                 % (self.__class__.__name__, `self.id()`, hex(id(self))[2:]))
 
-    security.declareProtected(Permissions.View, 'page_url')
-    def page_url(self):
-        """return the url path for this wiki page"""
+    security.declareProtected(Permissions.View, 'pageUrl')
+    def pageUrl(self):
+        """Return the url for this wiki page."""
         return self.wiki_url() + '/' + quote(self.id())
 
-    pageUrl=page_url
+    page_url=pageUrl
 
-    security.declareProtected(Permissions.View, 'wiki_url')
-    def wiki_url(self):
-        """return the base url path for this wiki"""
+    security.declareProtected(Permissions.View, 'wikiUrl')
+    def wikiUrl(self):
+        """Return the url for this wiki's folder."""
         try: return self.folder().absolute_url()
         except (KeyError,AttributeError): return '' # for debugging/testing
 
-    wikiUrl=wiki_url
+    wiki_url=wikiUrl
+
+    security.declareProtected(Permissions.View, 'wikiPath')
+    def wikiPath(self):
+        """Return the path part of this wiki's url.
+        """
+        # absolute_url_path and virtual_url_path just don't work 
+        # in a apache proxy-vhm-zope situation, apparently.
+        #try: return self.folder().absolute_url_path()
+        try: return re.sub(r'.*?//.*?/',r'/',self.folder().absolute_url())
+        except (KeyError,AttributeError): return '' # for debugging/testing
 
     def defaultPageUrl(self):
         p = self.defaultPage()
@@ -339,7 +352,7 @@ class Utils:
 
     security.declareProtected(Permissions.View, 'discussionUrl')
     def discussionUrl(self):
-        p = self.pageWithName('GeneralDiscussion')
+        p = self.pageWithName('UserDiscussion') or self.pageWithName('GeneralDiscussion')
         return (p and p.pageUrl()) or ''
 
     security.declareProtected(Permissions.View, 'trackerUrl')
@@ -350,6 +363,10 @@ class Utils:
     def filterUrl(self):
         return self.urlForDtmlPageOrMethod('FilterIssues','filterissues')
 
+    security.declareProtected(Permissions.View, 'filterUrl')
+    def issueBrowserUrl(self):
+        return self.urlForDtmlPageOrMethod('IssueBrowser','issuebrowser')
+
     security.declareProtected(Permissions.View, 'indexUrl')
     def indexUrl(self):
         return self.urlForDtmlPageOrMethod('AllPages','') #'allpages')
@@ -429,6 +446,14 @@ class Utils:
         return int(self.getPhysicalRoot().ZopeTime() -
                    self.lastEditTime())
 
+    security.declareProtected(Permissions.View, 'lastEditInterval')
+    def lastEditIntervalInHours(self):
+        """
+        return the number of hours since last edit
+        """
+        return int((self.getPhysicalRoot().ZopeTime() -
+                    self.lastEditTime()) * 24)
+
     security.declareProtected(Permissions.View, 'asAgeString')
     def asAgeString(self,time):
         """
@@ -476,7 +501,7 @@ class Utils:
         return s
 
     security.declareProtected(Permissions.View,'include')
-    def include(self,page,REQUEST=None):
+    def include(self,page,REQUEST=None, **kw):
         """
         Convenience method for including the body of one page within another.
 
@@ -484,7 +509,7 @@ class Utils:
         needed, fails silently if page does not exist.
         """
         p = self.pageWithNameOrId(page)
-        if p: return p(bare=1,REQUEST=REQUEST)
+        if p: return p(bare=1,REQUEST=REQUEST, **kw)
         else: return ''
         
 InitializeClass(Utils)
@@ -496,6 +521,11 @@ def BLATHER(*args):
     for arg in args: tmp.append(str(arg))
     zLOG.LOG('ZWiki',zLOG.BLATHER,' '.join(tmp))
 
+def WARN(*args):
+    tmp = []
+    for arg in args: tmp.append(str(arg))
+    zLOG.LOG('ZWiki',zLOG.WARNING,' '.join(tmp))
+
 def formattedTraceback():
     type,val,tb = sys.exc_info()
     try:     return join(traceback.format_exception(type,val,tb),'')
