@@ -75,6 +75,7 @@ from CMF import CMFAwareness
 from Comments import CommentsSupport
 from Admin import AdminSupport
 from Editing import EditingSupport
+from RSS import RSSMixin
 from I18nSupport import DTMLFile, _
 from pagetypes import PAGETYPES, PAGE_TYPES
 from pagetypes.common import MIDSECTIONMARKER
@@ -95,6 +96,7 @@ class ZWikiPage(
     CommentsSupport,
     AdminSupport,
     Utils,
+    RSSMixin,
     PLUGINS[0],     # runtime mixins hack, see plugins
     PLUGINS[1],
     PLUGINS[2],
@@ -570,8 +572,9 @@ class ZWikiPage(
                  self.bracketLinksAllowed())):
                 # yes - convert to the id of an existing page if possible,
                 # using fuzzy matching, and continue. 
-                p = self.pageWithFuzzyName(linknobrackets) or \
-                    self.issuePageWithNumber(self.issueNumberFrom(linknobrackets))
+                p = self.pageWithFuzzyName(linknobrackets)
+                    #or self.issuePageWithNumber(self.issueNumberFrom(linknobrackets))
+                    #keep freeform links to old IssueNo pages working, no longer needed
                 if p:
                     try: link = p.getId() # XXX poor caching
                     except: link = p.id # all-brains
@@ -1013,18 +1016,26 @@ class ZWikiPage(
     security.declareProtected(Permissions.View, 'defaultPage')
     def defaultPage(self):
         """
-        Return this wiki's default page (object).
+        Return this wiki's default/front page object.
 	
-	That is the page named in the default_page property,
-	or FrontPage,
-        or the first page object in the folder, 
-	or None.
+	That is:
+        -a page named in the default_page string or lines property
+	-or the page named FrontPage
+        -or the first page object in the folder
+	-or None.
         """
-        # XXX need to handle a plone-like list property also
-        # XXX and will this pick up default_page from portal_properties ?
-        return (
-            self.pageWithName(getattr(self.folder(),'default_page','FrontPage'))
-            or (list(self.pageObjects())+[None])[0]) # pageObjects may be a LazyMap
+        # default_page property could be a list, tuple or string
+        default_page_names = getattr(self.folder(),'default_page',[])
+        if type(default_page_names) == StringType: 
+            default_page_names = [default_page_names]
+        elif type(default_page_names) == TupleType:
+            default_page_names = list(default_page_names)
+        default_page_names.append('FrontPage')
+        for name in default_page_names:
+            p = self.pageWithName(name)
+            if p: return p
+        # pageObjects could be a LazyMap
+        return (list(self.pageObjects())+[None])[0]
         
     security.declareProtected(Permissions.View, 'defaultPageId')
     def defaultPageId(self):
