@@ -1,5 +1,6 @@
 from common import *
 from Products.ZWiki.I18nSupport import _
+from Products.ZWiki.pagetypes import registerPageType
 
 class ZwikiHtmlPageType(AbstractHtmlPageType):
     _id = 'html'
@@ -12,7 +13,7 @@ class ZwikiHtmlPageType(AbstractHtmlPageType):
         t = text or (page.read()+'\n'+MIDSECTIONMARKER)
         t = page.applyWikiLinkLineEscapesIn(t)
         t = page.markLinksIn(t)
-        t = self.escapeEmailAddresses(page,t)
+        t = self.protectEmailAddresses(page,t)
         return t
 
     def render(self, page, REQUEST={}, RESPONSE=None, **kw):
@@ -21,7 +22,8 @@ class ZwikiHtmlPageType(AbstractHtmlPageType):
         else:
             t = page.preRendered()
         t = page.renderMarkedLinksIn(t)
-        t = page.renderMidsectionIn(t)
+        t = page.renderMidsectionIn(t,**kw)
         t = page.addSkinTo(t,**kw)
         return t
 
+registerPageType(ZwikiHtmlPageType)
