@@ -1,5 +1,6 @@
 from common import *
 from Products.ZWiki.I18nSupport import _
+from Products.ZWiki.pagetypes import registerPageType
 
 class ZwikiPlaintextPageType(AbstractPageType):
     _id = 'plaintext'
@@ -12,12 +13,13 @@ class ZwikiPlaintextPageType(AbstractPageType):
         t = text or page.read()
         t = self.renderPlaintextIn(t)
         if not text: t += '\n'+MIDSECTIONMARKER
-        t = self.escapeEmailAddresses(page,t)
+        t = self.protectEmailAddresses(page,t)
         return t
 
     def render(self, page, REQUEST={}, RESPONSE=None, **kw):
         t = page.preRendered()
-        t = page.renderMidsectionIn(t)
+        t = page.renderMidsectionIn(t,**kw)
         t = page.addSkinTo(t,**kw)
         return t
 
+registerPageType(ZwikiPlaintextPageType)
